// プロトタイプ  (c)2014 Ryosuke Kawano

#include <avr/sleep.h>

// 定数を定義する
const int yellow_led = 13;    // D13 に黄色LEDを接続
const int face_sw = 2;        // D2 にスイッチを接続
const int ext_int = 0;        // 外部割り込み0 は D2と兼用
const int sounder = 3;        // D3 に圧電サウンダを接続
const int snd_freq = 1000;    // プー音の周波数 Hz
const int snd_time = 200;     // プー音の継続時間 ms
const int wait_snd = 400;     // プー音の待ち時間 ms（snd_timeの2倍）
const int wait_time = 15000;  // 状態遷移の待ち時間 ms
const int tick_time = 10;     // 刻み時間 ms

// 変数を定義する
int c_timer = 0;              // タイマカウンタ
int state = 0;                // 状態番号
                              // 0: 初期化状態
                              // 1: スリープ状態
                              // 2: 殴られ１回目状態

//--------------------------------------------------------------------
void setup( ){                
  // 初期化状態
  pinMode( face_sw, INPUT_PULLUP );
  pinMode( yellow_led, OUTPUT );
  bitClear( ADCSRA, ADEN );  // A/D変換禁止
  bitSet( PRR, PRADC );      // A/D変換器電力削減
  set_sleep_mode( SLEEP_MODE_PWR_DOWN );
  // 【条件１】初期化終了・・・・・成立
  state = 1;          // スリープ状態へ遷移
}

//--------------------------------------------------------------------
void loop( ){
  switch( state ){
  case 1:             // スリープ状態
    Sleep( );         // スリープ状態処理
    break;
  case 2:             // 殴られ１回目状態
    Nagurare1( );     // 殴られ１回目状態処理
    break;
  default:            //（フェールセーフ）
    state = 1;
  }
}

//--------------------------------------------------------------------
void Sleep( ){        // スリープ状態処理
  digitalWrite( yellow_led, LOW );
  attachInterrupt( ext_int, ir_wakeup, FALLING );
  sleep_mode( );    // CPU停止、外部割り込みを待つ
  // 【条件２】顔スイッチ オフ（HIGH）→オン（LOW）・・・・・成立
  detachInterrupt( ext_int );
  digitalWrite( yellow_led, HIGH );
  tone( sounder, snd_freq, snd_time );
  delay( wait_snd );
  c_timer = 0;
  state = 2;  // 殴られ１回目状態へ遷移
}

//--------------------------------------------------------------------
void Nagurare1( ){     // 殴られ１回目状態処理
  // この状態で行うことがある場合はここに書く
  delay( tick_time );
  c_timer++;
  // 【条件４】タイマカウンタが１５秒経過
  if ( c_timer >= ( wait_time / tick_time ) ){
    state = 1;  // スリープ状態へ遷移する
  }
}

//--------------------------------------------------------------------
void ir_wakeup( ){  // スリープモードを抜けたときに実行される外部割り込み
  // 割り込み内の処理なし
  ;
}

