/* スリープテストプログラム3  (c)2014 Ryosuke Kawano
  1. ハードウェアは ATmega328P マイコンとブレッドボードを利用する。
  2. マイコンのクロックは外付けセラミック発振子16MHz とする。
  3. 入力装置はデジタル入力D2 に相当するピンに接続したプッシュスイッチ 1個。
  4. プッシュスイッチは押していないとき HIGH、押したとき LOW とする。
  5. 出力装置はデジタル出力D13 に相当するピンに接続したLED 1個。
  6. LED の出力は LOW で消灯、HIGH で点灯とする。
  7. マイコンの初期設定では、A/Dコンバータを停止し、電力を低減させる。
  8. マイコンが動き始めて初期設定が終わると、以下の動作を行う。
  9. LEDを消灯したまま外部割り込みを許可してスリープする。
 10. プッシュスイッチを押すとウェイクアップし、外部割り込みを禁止する。
 11. 3秒間LED を点灯する。
 12. 9以降繰り返し。
 メモ：実測でスリープ中 19.3μA（LR44 ×3個）ただしヒューズビットはノーマル
*/

#include <avr/sleep.h>

const int led = 13;
const int sw = 2;            // D2 push sw
const int ext_int = 0;       // exrernal int 0 : D2
const int wait_time = 3000;

void setup( ){
  // 初期化状態
  pinMode( sw, INPUT_PULLUP );
  pinMode( led, OUTPUT );
  bitClear( ADCSRA, ADEN );  // A/D変換禁止
  bitSet( PRR, PRADC );      // A/D変換器電力削減
  set_sleep_mode( SLEEP_MODE_PWR_DOWN );
  // 【条件１】初期化終了が成立
}
void loop( ){
  // スリープ状態
  digitalWrite( led, LOW );
  attachInterrupt( ext_int, ir_wakeup, FALLING );
  sleep_mode( );
  // sw off(HIGH) --> on(LOW)
  // 【条件２】プッシュスイッチが押された
  // ウェイクアップ状態
  detachInterrupt( ext_int );
  digitalWrite( led, HIGH );
  delay( wait_time );
}

void ir_wakeup( ){
  ;
}

